@@ -2,6 +2,7 @@ import PyQt5.QtWidgets as qtWidget
 import PyQt5.QtGui as qtGui
 from PyQt5.QtCore import Qt
 import sys
+import scipy as sc
 import pandas as pd
 import numpy as np
 import statistics
@@ -9,10 +10,13 @@ from pathlib import Path
 import time
 import matplotlib.pyplot as plt
 from threading import Thread
-from queue import Queue
+from multiprocessing import Process, Queue
+# from queue import Queue
 from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
 from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
 import copy
+import re
+import posNegEvolution.externalPlots as externalPlots
 
 from posNegEvolution.pos_neg_evolution_init import posNegEvolutionMainLoop as pnEML 
 
@@ -34,6 +38,9 @@ class mainFormat(qtWidget.QWidget):
         self.s_ID = []
         self.q = Queue()
         self.createMainView()
+        self._th_hist = Thread()
+        self._th_mw = Thread()
+        self._th_fw = Thread()
         self._monitor = True
         self.th_monitor = Thread(target=self.monitor, args=(self.q, self.status))
         self.th_monitor.start()
@@ -74,7 +81,10 @@ class mainFormat(qtWidget.QWidget):
         while self._monitor:
             if not q.empty():
                 data = q.get()
-                if data[0] == '0':
+                if data[0] == 'exit':
+                    print('exit monitor')
+                    break
+                elif data[0] == '0':
                     status.setText(data[1] + ": " + data[2])
                 elif data[0] == '-1':
                     status.setText(data[1] + ": correlation plot")
@@ -84,6 +94,22 @@ class mainFormat(qtWidget.QWidget):
                     self.plot(data[1], data[2], 1)
                 else:
                     q.put(data)
+                    
+            if self._th_hist.is_alive():
+                self._th_hist.join(1)     
+                if not self._th_hist.is_alive():               
+                    self.status.setText("histograms done")
+                    # self._th_hist = None
+            if self._th_mw.is_alive():
+                self._th_mw.join(1)   
+                if not self._th_mw.is_alive():                 
+                    self.status.setText("mutation wave done")
+                    # self._th_mw = None
+            if self._th_fw.is_alive():
+                self._th_fw.join(1) 
+                if not self._th_fw.is_alive():                   
+                    self.status.setText("fitness wave done")
+                    # self._th_fw = None
     
     def about(self):
         global disclaimer
@@ -99,11 +125,18 @@ class mainFormat(qtWidget.QWidget):
             i.join()
         for i in self.th_s:
             i.join()
-            
-        self._monitor = False
+        
+        if self._th_hist.is_alive():
+            self._th_hist.join()
+        if self._th_mw.is_alive():
+            self._th_mw.join()
+        if self._th_fw.is_alive():
+            self._th_fw.join()    
+        
         plt.close(self._fig)
+        self.q.put(['exit'])
         self.th_monitor.join()
-        event.accept()  
+        event.accept()   
     
     def update(self):
         i = self.tabs.indexOf(self._tabUI)
@@ -163,8 +196,10 @@ class mainFormat(qtWidget.QWidget):
         layout.addWidget(self.tabs)
         layout.addStretch()
         
-        self._binned = qtWidget.QCheckBox("Checked: binned version")
-        layout.addWidget(self._binned)
+        self._VAF = qtWidget.QCheckBox("Checked: Include VAF")
+        layout.addWidget(self._VAF)
+        self._analytical = qtWidget.QCheckBox("Checked: Analytical model (omit VAF)")
+        layout.addWidget(self._analytical)        
         
         start = qtWidget.QPushButton(self)
         start.setText("Start simulation")
@@ -210,7 +245,51 @@ class mainFormat(qtWidget.QWidget):
 
         self.threadTab.setLayout(_threadTab)
         return self.threadTab        
-        
+    
+    def cloneHistAction(self):
+        fname = qtWidget.QFileDialog.getOpenFileNames(None, 'Open file', "Z://","Data File (*.npz)")[0] 
+        if len(fname) == 0:
+            self.showDialog("No file selected!", "Alert")
+            return
+        name_id = []
+        for i in fname:
+            name_id.append(re.findall('\d+', i)[-1])
+        if all(map(lambda x: x.endswith('.npz'), fname)):
+            self._th_hist = (Thread(target=externalPlots.VAFplot, args=(fname)))
+            # external_plots.binnedHist(fname, self._file_path.text(), name_id)
+            self._th_hist.start()
+            self.status.setText("mutations histograms ongoing")
+        else:
+            self.showDialog("Wrong file/files extension. Use only one kind at time", "Alert")
+            
+    def mutWaveAction(self):
+        fname = qtWidget.QFileDialog.getOpenFileNames(None, 'Open file', "Z://","Data File (*.npz)")[0] 
+        if len(fname) == 0:
+            self.showDialog("No file selected!", "Alert")
+            return
+        name_id = []
+        for i in fname:
+            name_id.append(re.findall('\d+', i)[-1])
+            
+        self._th_mw = (Thread(target=externalPlots.mutationWave, args=(fname)))
+        # external_plots.mutWavePlot(fname, self._file_path.text(), name_id)
+        self._th_mw.start()
+        self.status.setText("mutation wave ongoing")
+    
+    def fitWaveAction(self):
+        fname = qtWidget.QFileDialog.getOpenFileNames(None, 'Open file', "Z://","Data File (*.npz)")[0] 
+        if len(fname) == 0:
+            self.showDialog("No file selected!", "Alert")
+            return
+        name_id = []
+        for i in fname:
+            name_id.append(re.findall('\d+', i)[-1])
+            
+        self._th_fw = (Thread(target=externalPlots.fitnessWave, args=(fname)))
+        # external_plots.fitWavePlot(fname, self._file_path.text(), name_id)
+        self._th_fw.start()
+        self.status.setText("fitness wave ongoing")
+    
     def generalTabUI(self):
         """Create the General page UI."""
         generalTab = qtWidget.QWidget()
@@ -228,6 +307,17 @@ class mainFormat(qtWidget.QWidget):
         self._file_desc = qtWidget.QLineEdit()
         _file_desc_label = qtWidget.QLabel("TODO")
         _file_desc_label.setText('Description')
+        
+        _clone_hist_button = qtWidget.QPushButton(self)
+        _clone_hist_button.setText('Mutations histograms')
+        _clone_hist_button.clicked.connect(self.cloneHistAction)
+        _mut_wave = qtWidget.QPushButton(self)
+        _mut_wave.setText('Mutation wave plot')
+        _mut_wave.clicked.connect(self.mutWaveAction)
+        _fit_wave = qtWidget.QPushButton(self)
+        _fit_wave.setText('Fitness wave plot')
+        _fit_wave.clicked.connect(self.fitWaveAction)
+        
         _file_instr_fn = qtWidget.QLabel()
         _file_instr_fn.setText('File name - one cycle file name will appear with cycle number')
         _file_instr_fp = qtWidget.QLabel()
@@ -235,16 +325,30 @@ class mainFormat(qtWidget.QWidget):
         _file_instr_fd = qtWidget.QLabel()
         _file_instr_fd.setText('Description - most important information to add to file')
         
-        _generalTab.addWidget(_file_name_label, 0, 0)
-        _generalTab.addWidget(self._file_name, 0, 1, 1, 2)
-        _generalTab.addWidget(_file_path_label, 1, 0)
-        _generalTab.addWidget(self._file_path, 1, 1)
-        _generalTab.addWidget(_file_path_button, 1, 2)
-        _generalTab.addWidget(_file_desc_label, 2, 0, 1, 3)
-        _generalTab.addWidget(self._file_desc, 3, 0, 2, 3)
-        _generalTab.addWidget(_file_instr_fn, 5, 0, 1, 3)
-        _generalTab.addWidget(_file_instr_fp, 6, 0, 1, 3)
-        _generalTab.addWidget(_file_instr_fd, 7, 0, 1, 3)
+        row = 0
+        _generalTab.addWidget(_file_name_label, row, 0)
+        _generalTab.addWidget(self._file_name, row, 1, 1, 2)
+        row = row + 1
+        _generalTab.addWidget(_file_path_label, row, 0)
+        _generalTab.addWidget(self._file_path, row, 1)
+        _generalTab.addWidget(_file_path_button, row, 2)
+        row = row + 1
+        _generalTab.addWidget(_file_desc_label, row, 0, 1, 3)
+        row = row + 1
+        _generalTab.addWidget(self._file_desc, row, 0, 1, 3)
+        row = row + 1
+        _generalTab.addWidget(_clone_hist_button, row, 0, 1, 3)
+        row = row + 1
+        _generalTab.addWidget(_mut_wave, row, 0, 1, 3)
+        row = row + 1
+        _generalTab.addWidget(_fit_wave, row, 0, 1, 3)
+        row = row + 1
+        _generalTab.addWidget(_file_instr_fd, row, 0, 1, 3)
+        row = row + 1
+        _generalTab.addWidget(_file_instr_fn, row, 0, 1, 3)
+        row = row + 1
+        _generalTab.addWidget(_file_instr_fp, row, 0, 1, 3)
+        row = row + 1
         
         generalTab.setLayout(_generalTab)
         return generalTab
@@ -286,8 +390,8 @@ class mainFormat(qtWidget.QWidget):
         self._skip = qtWidget.QLineEdit(str(10))
         self._skip.setValidator(qtGui.QDoubleValidator(0.1, 100.0, 1))
         
-        self._mut_effect = qtWidget.QLineEdit(".1, -.001")
-        self._mut_effect.setInputMask("[#.#####, #.#####]")
+        self._mut_effect = qtWidget.QLineEdit(".001, -.001")
+        self._mut_effect.setInputMask("[##.#####, ##.#####]")
         self._mut_prob = qtWidget.QLineEdit("0.0001, 0.05")        
         self._mut_prob.setInputMask("[#.#####, #.#####]")
         
@@ -329,7 +433,7 @@ class mainFormat(qtWidget.QWidget):
         row = row + 1
         _parametersTab.addWidget(qtWidget.QLabel("Simulation steps"), row, 0)
         _parametersTab.addWidget(self._steps, row, 1)
-        _parametersTab.addWidget(qtWidget.QLabel("min: 0, max: 10^5"), row, 2)
+        _parametersTab.addWidget(qtWidget.QLabel("number of steps,(optional: 's' as type of break - population size)"), row, 2)
         row = row + 1
         _parametersTab.addWidget(qtWidget.QLabel("Tau step"), row, 0)
         _parametersTab.addWidget(self._tau, row, 1)
@@ -379,7 +483,7 @@ class mainFormat(qtWidget.QWidget):
         self.status.setText("Critical Population: " + str(round((mut_prob[1]/mut_prob[0])*(mut_effect[1]/mut_effect[0]**2),0)))
     
     def selectPath(self):
-        dir_path = qtWidget.QFileDialog.getExistingDirectory(self,"Choose Directory","Z:\\")
+        dir_path = qtWidget.QFileDialog.getExistingDirectory(self,"Choose Directory","Z:/")
         self._file_path.setText(dir_path)
 
     def saveParams(self):
@@ -420,15 +524,21 @@ class mainFormat(qtWidget.QWidget):
                 'lambda': iLambda
             }, index=[0])
         
-        filepath = Path(self._file_path.text() + '\\params'  + ".csv")  
+        filepath = Path(self._file_path.text() + '/params'  + ".csv")  
         filepath.parent.mkdir(parents=True, exist_ok=True)  
         dfp.to_csv(filepath)  
 
     def simStart(self):
+        break_type = 0
         try:
             pop = int(self._population.text())
             cap = int(self._capacity.text())
-            steps = int(self._steps.text())
+            steps = self._steps.text().split(',')
+            if len(steps) == 2:
+                steps = int(steps[0])
+                break_type = 1
+            else:
+                steps = int(steps[0])
             tau = float(self._tau.text())
             skip = float(self._skip.text())
             mut_prob = self._mut_prob.text().split(',')
@@ -451,45 +561,64 @@ class mainFormat(qtWidget.QWidget):
             return
         
         plots = 1*self._mw.isChecked() + 2*self._fw.isChecked() + 16*self._sx.isChecked()
+        iPop = []
+        select = 0
         
-        if self._binned.isChecked():
-            self._status.setText("binned version not available")
+        if self._analytical.isChecked():
+            iPop = np.array([[0,0],[0,pop]])
+            iPop = sc.sparse.csr_matrix(iPop)
+            select = 2
+        elif self._VAF.isChecked():
+            iPop = np.ones(pop)
+            iMuts = np.zeros(pop)
+            iPop = sc.sparse.csr_matrix(np.array([iPop, iMuts]).T)
+            select = 1
         else:        
             iMuts = np.zeros(pop, dtype=np.int64).tolist()
             iProp = np.ones(pop).tolist()
+            iPop = sc.sparse.csr_matrix(np.array([iProp, iMuts, iMuts]).T)
             
-            self.status.setText("Started")
-            self.status.setStyleSheet("background-color: green")
-            time.sleep(1)
-    
-            self.th_s.append(Thread(target=pnEML, args=(np.array([
-                                                        copy.deepcopy(iProp), 
-                                                        copy.deepcopy(iMuts), 
-                                                        copy.deepcopy(iMuts)]).T.tolist(), 
-                                                        copy.deepcopy([pop, cap, steps, tau, skip, mut_prob, mut_effect, iLambda, threads]), 
-                                                        self._file_name.text(), 
-                                                        self._file_desc.text(), 
-                                                        self._file_path.text(), plots, 0, self.q, self.ID, 0)))    
+        self.status.setText("Started")
+        self.status.setStyleSheet("background-color: green")
+        time.sleep(1)
 
-            self.s_ID.append(str(self.ID))
-            self.ID = self.ID + 1
-            self.th_s[self.idx_s].start()
-            self.idx_s = self.idx_s + 1
+        self.th_s.append(Process(target=pnEML, args=(copy.deepcopy(iPop), 
+                                                    copy.deepcopy([pop, cap, steps, tau, skip, mut_prob, mut_effect, iLambda, threads]), 
+                                                    self._file_name.text(), 
+                                                    self._file_desc.text(), 
+                                                    self._file_path.text(), plots, 0, self.q, self.ID, select, break_type))) 
+        
+        # pnEML(copy.deepcopy(iPop),
+        #     copy.deepcopy([pop, cap, steps, tau, skip, mut_prob, mut_effect, iLambda, threads]), 
+        #     self._file_name.text(), 
+        #     self._file_desc.text(), 
+        #     self._file_path.text(), plots, 0, self.q, self.ID, select)      
+        
+        self.s_ID.append(str(self.ID))
+        self.ID = self.ID + 1
+        self.th_s[self.idx_s].start()
+        self.idx_s = self.idx_s + 1
        
     def resume(self):
         self.showPromptParams()
-        fname = qtWidget.QFileDialog.getOpenFileName(None, 'Open file', "Z://","CSV files (*.csv);; Text files (*.txt)")[0] 
+        fname = qtWidget.QFileDialog.getOpenFileName(None, 'Open file', "Z://","Data files (*.npz)")[0] 
         if fname == "":
             self.showDialog("No file selected!", "Alert")
             return
         self.loadPaths(fname)
         
+        break_type = 0
         try:
             pop = int(self._population.text())
-            cap = int(self._population.text())
-            steps = int(self._population.text())
-            tau = float(self._population.text())
-            skip = float(self._population.text())
+            cap = int(self._capacity.text())
+            steps = self._steps.text().split(',')
+            if len(steps) == 2:
+                steps = int(steps[0])
+                break_type = 1
+            else:
+                steps = int(steps[0])
+            tau = float(self._tau.text())
+            skip = float(self._skip.text())
             mut_prob = self._mut_prob.text().split(',')
             mut_prob = [float(x.strip('[]')) for x in mut_prob]
             mut_effect = self._mut_effect.text().split(',')
@@ -498,31 +627,29 @@ class mainFormat(qtWidget.QWidget):
             threads = int(self._threads.text())
         except:
             self.showDialog("Type correct parameters","Alert")
-            return        
-           
-        df = pd.read_csv(fname)
+            return   
         
-        plots = 1*self._mw.isChecked() + \
-                2*self._fw.isChecked() + \
-                16*self._sx.isChecked()
-        
-        iProp = df['Fitness'].tolist()
-        ipMutations = df['Positive mutations'].tolist()
-        inMutations = df['Negative mutations'].tolist()
-        
-        del df
+        plots = 1*self._mw.isChecked() + 2*self._fw.isChecked() + 16*self._sx.isChecked()
+        iPop = []
+        select = 0
+        if self._analytical.isChecked():
+            iPop = sc.sparse.load_npz(fname)
+            select = 2
+        elif self._VAF.isChecked():
+            iPop = sc.sparse.load_npz(fname)
+            select = 1
+        else:        
+            iPop = sc.sparse.load_npz(fname)
                 
         self.status.setText("Started")
         self.status.setStyleSheet("background-color: green")
         time.sleep(1)
         
-        self.th_r.append(Thread(target=pnEML, args=(([copy.deepcopy(iProp), 
-                                                    copy.deepcopy(ipMutations), 
-                                                    copy.deepcopy(inMutations)]).T.tolist(), 
-                                                   copy.deepcopy([pop, cap, steps, tau, skip, mut_prob, mut_effect, iLambda, threads]), 
-                                                   self._file_name.text(), 
-                                                   self._file_desc.text(), 
-                                                   self._file_path.text(), plots, self._last_cycle+1, self.q, self.ID, 0)))
+        self.th_r.append(Process(target=pnEML, args=(copy.deepcopy(iPop), 
+                                                    copy.deepcopy([pop, cap, steps, tau, skip, mut_prob, mut_effect, iLambda, threads]), 
+                                                    self._file_name.text(), 
+                                                    self._file_desc.text(), 
+                                                    self._file_path.text(), plots, 0, self.q, self.ID, select, break_type))) 
         self.r_ID.append(str(self.ID))
         self.ID = self.ID + 1
         self.th_r[self.idx_r].start()
@@ -605,7 +732,7 @@ class mainFormat(qtWidget.QWidget):
                 elif i.endswith('.txt'):
                     name = i.rstrip('_' + str(self._last_cycle) + '.txt')
             else:
-                path = path + i + '\\'
+                path = path + i + '/'
         
         if name.endswith("_binned"):
             name = name.rstrip("_binned")
